@@ -353,6 +353,7 @@ class ALU_Dataset():
 
     if 'tensorflow' in sys.modules:
         def create_dataset(self, samples=10000, batch_size=2000, vector=False, positional_encoding=True, is_training=True, valid_ops=None, name=None, cache_path=None, use_cache=True, regenerate_cached_data=False, for_tpu=False):
+            import tensorflow as tf
             is_loaded=False
             if use_cache is True and cache_path is None:
                 print("can't use cache if no cache_path is given, disabling cache!")
@@ -420,6 +421,7 @@ class ALU_Dataset():
             return dataset
 
         def create_dataset_from_generator(self, valid_ops=None):
+            import tensorflow as tf
             dataset=tf.data.Dataset.from_generator(
                 self._generator,
                 output_signature=(
@@ -428,9 +430,9 @@ class ALU_Dataset():
                 )
             return dataset
 
-        def get_datasets(self, pre_weight=True, samples=100000, validation_samples=10000, batch_size=2000, vector=False, positional_encoding=True, valid_ops=None, cache_path=None, use_cache=True, regenerate_cached_data=False):
+        def get_datasets(self, pre_weight=True, samples=100000, validation_samples=10000, batch_size=2000, vector=False, positional_encoding=True, valid_ops=None, cache_path=None, use_cache=True, regenerate_cached_data=False, for_tpu=False):
             train = self.create_dataset(samples=samples, batch_size=batch_size, is_training=True, vector=vector, positional_encoding=positional_encoding, valid_ops=valid_ops,
-                                            name="train",cache_path=cache_path, use_cache=use_cache, regenerate_cached_data=regenerate_cached_data)
+                                            name="train",cache_path=cache_path, use_cache=use_cache, regenerate_cached_data=regenerate_cached_data, for_tpu=for_tpu)
             val = self.create_dataset(samples=validation_samples, batch_size=batch_size, vector=vector, positional_encoding=positional_encoding, is_training=False, valid_ops=valid_ops,
                                         name="validation",cache_path=cache_path, use_cache=use_cache, regenerate_cached_data=regenerate_cached_data)
             return train, val
