@@ -22,13 +22,18 @@ class Text_Dataset:
         td = Text_Dataset(ls)
     
     :param text_list: list of text-records of the form: {'author': 'author', 'title': 'title', 'language': 'some-language', 'text': 'the-long-text'}. Optinal parameters: 'weight': 1.0
+    :param sanitize_white_space: If True, white space is replaced by a single space.
+    :param separate_punctuation: If True, punctuation is separated from words.
+    :param preserve_case: If True, the case of the text is preserved.
     """
-    def __init__(self, text_list):
+    def __init__(self, text_list, sanitize_white_space=False, separate_punctuation=False, preserve_case=True):
         self.log = logging.getLogger("Datasets")
         self.text_list = []
         self.index = 1
         self.word_tokenizer_init = False
         self.char_tokenizer_init = False
+        self.getitem_init = False
+        
         req_attrs=['title', 'author', 'language', 'text']
         for ind in range(0,len(text_list)):
             valid=True
@@ -42,6 +47,7 @@ class Text_Dataset:
                 continue
             text=text_list[ind]
             text['index']=self.index
+            text['text']=self.filter_text(text['text'], sanitize_white_space=sanitize_white_space, separate_punctuation=separate_punctuation, preserve_case=preserve_case)
             self.index += 1
             self.text_list.append(text)
         self.log.info(f"Loaded {len(self.text_list)} texts")
@@ -79,7 +85,7 @@ class Text_Dataset:
         else:
             return random.choice(self.tidx)
 
-    def filter_text(self, text, sanitize_white_space=True, separate_punctuation=False, preserve_case=True):
+    def filter_text(self, text, sanitize_white_space=False, separate_punctuation=False, preserve_case=True):
         """ Filter a text.
         
         :param text: text to filter
@@ -106,36 +112,31 @@ class Text_Dataset:
                 text = text.replace('  ', ' ')
         return text
 
-    def get_random_sample(self, length, weighted=True, sanitize_white_space=True, separate_punctuation=False, preserve_case=True):
+    def get_random_sample(self, length, weighted=True):
         """ Return index idx and random sample of `length` chars from text[idx] the Text_Dataset.
         
         :param length: number of characters to return
-        :param weighted: If True, the probability of a text is weighted by its calculated 'probability_weight' attribute.
         :param sanitize_white_space: If True, white space is replaced by a single space.
         :param separate_punctuation: If True, punctuation is separated from words.
         :param preserve_case: If True, the case of the text is preserved.
+        :param weighted: If True, the probability of a text is weighted by its calculated 'probability_weight' attribute.
         :return: tuple (idx of text used for sampling, string of length `length` sampled from the Text_Dataset)
         """
         idx = self._get_random_text_index(weighted)
         text = self.text_list[idx]['text']
         if len(text) < length:
             sample = text
+            while len(sample) < length:
+                sample += ' '
         else:
             pos = random.randint(0, len(text) - length)
             sample = text[pos:pos+length]
-        if sanitize_white_space is True:
-            sample = self.filter_text(sample, sanitize_white_space=sanitize_white_space, separate_punctuation=separate_punctuation, preserve_case=preserve_case)
         return (idx, sample)
 
     def _word_splitter(self, text):
-        txt=self.filter_text(text, sanitize_white_space=True, separate_punctuation=True, preserve_case=True)
-        tokens=txt.split()
+        tokens=text.split()
         return tokens
 
-    def _char_filter(self, text):
-        txt=self.filter_text(text, sanitize_white_space=True, separate_punctuation=False, preserve_case=True)
-        return txt
-
     def init_tokenizer(self, tokenizer='word'):
         """ Initialize the tokenizer with the text_list.
         
@@ -162,21 +163,22 @@ class Text_Dataset:
         elif tokenizer == 'char':
             self.i2c = {}
             self.c2i = {}
-            self.i2c['<unk>'] = 0
-            self.c2i[0] = '<unk>'
-            self.i2c['<pad>'] = 1
-            self.c2i[1] = '<pad>'
-            self.i2c['<eos>'] = 2
-            self.c2i[2] = '<eos>'
-            self.i2c['<sos>'] = 3
-            self.c2i[3] = '<sos>'
+            self.c2i['<unk>'] = 0
+            self.i2c[0] = '<unk>'
+            self.c2i['<pad>'] = 1
+            self.i2c[1] = '<pad>'
+            self.c2i['<eos>'] = 2
+            self.i2c[2] = '<eos>'
+            self.c2i['<sos>'] = 3
+            self.i2c[3] = '<sos>'
             for text in self.text_list:
-                txt = self._char_filter(text['text'])
-                uniq_chars = set(txt)
-                for c in uniq_chars:
+                tokens = list(text['text'])
+                unique_chars = set(tokens)
+                for c in unique_chars:
                     if c not in self.c2i:
-                        self.c2i[c] = len(self.c2i)
-                        self.i2c[len(self.i2c)] = c
+                        ind = len(self.i2c)
+                        self.c2i[c] = ind
+                        self.i2c[ind] = c
             self.char_tokenizer_init=True
         else:
             self.log.error(f"Unknown tokenizer {tokenizer}")
@@ -195,7 +197,7 @@ class Text_Dataset:
         elif tokenizer == 'char':
             if self.char_tokenizer_init is False:
                 self.init_tokenizer(tokenizer)
-            tokens = list(self._char_filter(text))
+            tokens = list(text)
         else:
             self.log.error(f"Unknown tokenizer {tokenizer}")
             raise ValueError(f"Unknown tokenizer {tokenizer}")
@@ -206,7 +208,7 @@ class Text_Dataset:
         
         :param text: text to encode
         :return: list of encoded tokens """
-        tokens = self.tokenize(text, tokenizer)
+        tokens = self.tokenize(text, tokenizer=tokenizer)
         if tokenizer == 'word':
             encoded = [self.w2i[token] if token in self.w2i else self.w2i['<unk>'] for token in tokens]
         elif tokenizer == 'char':
@@ -238,12 +240,115 @@ class Text_Dataset:
         :param length: length of the sample
         :return: tuple (X, y) encoded sample
         """
-        _, sample = self.get_random_sample(length+1, sanitize_white_space=False)
+        _, sample = self.get_random_sample(length+1)
         e_sample = self.encode(sample, tokenizer='char')
         X = e_sample[:-1]
         y = e_sample[1:]
         return X, y
-        
+
+    def init_getitem(self, sample_type='chargen', sample_length=80, content_stepping=10):
+        """ Initialize the __getitem__ and __len__ methods.
+
+        This method needs to be called before using len() or index-access of the dataset.
+
+        This method determines how the dataset is partitioned into records, and what kind
+        of encoding is returned on index-access.
+
+        .. code-block:: python
+
+            from ml_indie_tools.Text_Dataset import TextDataset
+            tl = [{'author':'nobody', 'title':'some title', 'language':'english', 'text':'some text'},
+                  {'author':'nobody', 'title':'some title 2', 'language':'english', 'text':'some more text'}]
+            td = Text_Dataset(tl)
+            td.init_getitem(sample_type='chargen', sample_length=4, content_stepping=2)
+            print(len(td))
+            print(td[0])
+            # Output: 12 and ('some', 'ome ')
+
+        :param sample_type: 'chargen': generate a pair of text or length sample_length, shifted by one letter, or 'chargen_encoded', same but encoded.
+        :param sample_length: length of a sample
+        :param content_stepping: number of characters to skip between each sample
+        :return: on 'chargen[_encoded]': X, y [encoded] strings of sample_length, y shifted by one letter.
+        """
+
+        self.getitem_sample_type = sample_type
+        self.getitem_sample_length = sample_length
+        self.getitem_content_stepping = content_stepping
+        leng=0
+        rec=0
+        if sample_type=='chargen' or sample_type=='chargen_encoded':
+            for ind in range(0, len(self.text_list)):
+                len_text = len(self.text_list[ind]['text'])
+                rec_text = (len_text-content_stepping+1)//content_stepping + 1
+                self.text_list[ind]['records']=rec_text
+                leng += len_text
+                rec += rec_text
+            self.getitem_length = leng
+            self.getitem_records = rec
+            self.getitem_init = True
+        else:
+            self.getitem_length = 0
+            self.getitem_records = 0
+            print(f"init_getitem: unknown sample_type {sample_type}")
+
+    def __len__(self):
+        """ Get the length of the dataset.
+
+        Note that this length depends on the initialization via :ref:`~Text_Dataset.Text_Dataset.init_getitem`.
+
+        :return: length of the dataset (mode dependent) 
+        """
+        if self.getitem_init is False:
+            print("init_getitem must be called before __len__")
+            return None
+        return self.getitem_records
+
+    def _getitem_chargen(self, index):
+        if index<0:
+            if index<(-self.getitem_records):
+                raise IndexError(f"index {index} out of range")
+            else:
+                index += self.getitem_records
+        if index >= self.getitem_records:
+            raise IndexError(f"index {index} out of range")
+        cur_rec = 0
+        for text in self.text_list:
+            rec = text['records']
+            if cur_rec+rec > index:
+                rel_rec = index - cur_rec
+                pos = rel_rec*self.getitem_content_stepping
+                sample = text['text'][pos:pos+self.getitem_sample_length+1]
+                while len(sample) < self.getitem_sample_length+1:
+                    sample += ' '
+                X_text = sample[:-1]
+                y_text = sample[1:]
+                return X_text, y_text
+            cur_rec += rec
+        print("Internal error in __getitem__")
+        raise ValueError("Internal error in __getitem__")
+
+    def __getitem__(self, index):
+        """ Get a sample from the dataset.
+
+        Format of the returned sample depends on :ref:`~Text_Dataset.Text_Dataset.init_getitem`.
+
+        :param index: index of the sample
+        :return:
+        """
+        if self.getitem_init is False:
+            print("init_getitem must be called before __getitem__")
+            raise ValueError("init_getitem must be called before __getitem__")
+        if self.getitem_sample_type == 'chargen':
+            return self._getitem_chargen(index)
+        elif self.getitem_sample_type == 'chargen_encoded':
+            X, y = self._getitem_chargen(index)
+            X = self.encode(X, tokenizer='char')
+            y = self.encode(y, tokenizer='char')
+            return X, y
+        else:
+            self.log.error(f"Unknown getitem sample_type {self.getitem_sample_type}")
+            raise ValueError(f"Unknown getitem sample_type {self.getitem_sample_type}")
+
     def _display_colored_html(self, textlist, dark_mode=False, display_ref_anchor=True, pre='', post=''):
         """ Internal function to display text and citation references in HTML. """
         bgcolorsWht = ['#d4e6e1', '#d8daef', '#ebdef0', '#eadbd8', '#e2d7d5', '#edebd0',
